@@ -69,6 +69,7 @@
 #undef STATIC_KERBEROS4
 #undef STATIC_LOGIN
 #undef STATIC_MYSQL
+#undef STATIC_NTLM
 #undef STATIC_OTP
 #undef STATIC_PLAIN
 #undef STATIC_SASLDB
@@ -77,9 +78,6 @@
 /* This is where plugins will live at runtime */
 #undef PLUGINDIR
 
-/* Do we need a leading _ for dlsym? */
-#undef DLSYM_NEEDS_UNDERSCORE
-
 /* Make autoheader happy */
 #undef WITH_SYMBOL_UNDERSCORE
 
@@ -165,7 +163,27 @@
 /* do we have sys/uio.h? */
 #undef HAVE_SYS_UIO_H
 
-/* define if your system has getnameinfo() */
+/* do we have sys/param.h? */
+#undef HAVE_SYS_PARAM_H
+
+/* do we have sysexits.h? */
+#undef HAVE_SYSEXITS_H
+
+/* stdarg.h? varargs.h? */
+#undef HAVE_STDARG_H
+#undef HAVE_VARARGS_H
+
+/* Do we need a leading _ for dlsym? */
+#undef DLSYM_NEEDS_UNDERSCORE
+
+/* Does libtool support shared libs on this system? */
+#undef HAVE_DLFCN_H
+#undef DO_DLOPEN
+
+/* Should we try to dlopen stuff when we are staticly compiled? */
+#undef TRY_DLOPEN_WHEN_STATIC
+
+/* define if your system has getaddrinfo() */
 #undef HAVE_GETADDRINFO
 
 /* define if your system has getnameinfo() */
@@ -235,7 +253,9 @@ struct iovec {
 #include <sys/socket.h>
 #ifndef WIN32
 # include <netdb.h>
-# include <sys/param.h>
+# ifdef HAVE_SYS_PARAM_H
+#  include <sys/param.h>
+# endif
 #else /* WIN32 */
 # include <winsock.h>
 #endif /* WIN32 */
@@ -271,8 +291,31 @@ struct sockaddr_storage {
 #include "gai.h"
 #endif
 
+/* Defined in RFC 1035. max strlen is only 253 due to length bytes. */
+#ifndef MAXHOSTNAMELEN
+#define        MAXHOSTNAMELEN  255
+#endif
+
+#ifndef HAVE_SYSEXITS_H
+#include "exits.h"
+#else
+#include "sysexits.h"
+#endif
+
 #ifndef	NI_WITHSCOPEID
 #define	NI_WITHSCOPEID	0
 #endif
 
+/* Get the correct time.h */
+#if TIME_WITH_SYS_TIME
+# include <sys/time.h>
+# include <time.h>
+#else
+# if HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
+
 #endif /* CONFIG_H */
